/* $Id: shhcards.c,v 1.3 2002/03/02 20:51:06 sverrehu Exp $ */
/*------------------------------------------------------------------------
 |  FILE            shhcards.c
 |  MODULE OF       XPM game card library.
 |
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>

#include "shhcards.h"

/*-----------------------------------------------------------------------+
|  PRIVATE DATA                                                          |
+-----------------------------------------------------------------------*/

/* pixmaps are static */
#include "all-cards.xpm"

static Display *cardDisplay = NULL;
static Drawable cardDrawable;
static GC       cardGC;
static int      cardClipMaskSet;
static Pixmap   cardPixmap;
static Pixmap   cardClipMask;

/*-----------------------------------------------------------------------+
|  PRIVATE FUNCTIONS                                                     |
+-----------------------------------------------------------------------*/

static void
cardSetupPixmaps(void)
{
    int err;
    XpmAttributes xa;

    xa.valuemask = 0;
    xa.closeness = 40000;
    xa.valuemask |= XpmCloseness;
    err = XpmCreatePixmapFromData(cardDisplay, cardDrawable,
				  allCards, &cardPixmap, &cardClipMask, &xa);
    if (err == XpmSuccess /* 0 */)
	return;
    else {
	fprintf(stderr, "Xpm: %s\n", XpmGetErrorString(err));
	if (err < 0)  /* real failure */
	    exit(1);
    }
}

static void
cardFreePixmaps(void)
{
    XFreePixmap(cardDisplay, cardClipMask);
    XFreePixmap(cardDisplay, cardPixmap);
}

static void
cardSetupGC(void)
{
    XGCValues values;

    cardGC = XCreateGC(cardDisplay, cardDrawable, 0UL, &values);
    XSetGraphicsExposures(cardDisplay, cardGC, 0);
    XSetClipMask(cardDisplay, cardGC, cardClipMask);
}

static void
cardFreeGC(void)
{
    XFreeGC(cardDisplay, cardGC);
}

/*-----------------------------------------------------------------------+
|  PUBLIC FUNCTIONS                                                      |
+-----------------------------------------------------------------------*/

/* d must be same depth and same visual class as the intended window. */
void
cardInit(Display *dpy, Drawable d)
{
    cardDisplay = dpy;
    cardDrawable = d;
    cardClipMaskSet = 0;
    cardSetupPixmaps();
    cardSetupGC();
}

void
cardFinish(void)
{
    if (cardDisplay == NULL)
	return;
    cardFreePixmaps();
    cardFreeGC();
}

/* value is 1 to 13 for normal cards, 0 to N for backs and outlines. */
void
cardDraw(Drawable d, int x, int y, int suit, int value)
{
    int localX, localY;

    if (suit == CARD_BACKGROUND || suit == CARD_OUTLINE) {
	localX = CARD_WIDTH * value;
	localY = CARD_HEIGHT * (13 + suit - CARD_BACKGROUND);
    } else {
	localX = CARD_WIDTH * suit;
	localY = CARD_HEIGHT * (value - 1);
    }
    XSetClipOrigin(cardDisplay, cardGC, x - localX, y - localY);
    XCopyArea(cardDisplay, cardPixmap, d, cardGC,
	      localX, localY, CARD_WIDTH, CARD_HEIGHT, x, y);
}
