/* $Id: tst.c,v 1.3 2002/03/02 20:51:06 sverrehu Exp $ */
/*------------------------------------------------------------------------
 |  FILE            tst.c
 |  MODULE OF       XPM card game library.
 |
 |  DESCRIPTION     Simple test program opening a window and drawing some
 |                  cards into it.
 |
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <X11/Xlib.h>

#include "shhcards.h"

/*-----------------------------------------------------------------------+
|  PRIVATE DATA                                                          |
+-----------------------------------------------------------------------*/

#define RANDOM(x) (random() % (x))
#define RANDOMIZE(x) (srandom((unsigned) (x)))

static Display *dpy;
static Window   win;
static GC       gc;
static int      width = 580, height = 435;

/*-----------------------------------------------------------------------+
|  PRIVATE FUNCTIONS                                                     |
+-----------------------------------------------------------------------*/

static void
winInit(void)
{
    XGCValues values;
    XColor xc, xc2, fgCol, bgCol;

    if ((dpy = XOpenDisplay(NULL)) == NULL) {
	fprintf(stderr, "cannot connect to X server %s\n", XDisplayName(NULL));
	exit(1);
    }

    XAllocNamedColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
		     "white", &xc, &xc2);
    fgCol = xc;
    XAllocNamedColor(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
		     "dark olive green", &xc, &xc2);
    bgCol = xc;
    win = XCreateSimpleWindow(dpy, RootWindow(dpy, DefaultScreen(dpy)),
			      0, 0, width, height, 4,
			      fgCol.pixel, bgCol.pixel);
    values.foreground = fgCol.pixel;
    values.background = bgCol.pixel;
    gc = XCreateGC(dpy, win, 0UL, &values);
    XMapWindow(dpy, win);
}

static void
doSomething(void)
{
    int q, suit, value, x, y;

    for (q = 0; q < 100; q++) {
	suit = RANDOM(4);
	value = 1 + RANDOM(13);
	x = RANDOM(width - CARD_WIDTH);
	y = RANDOM(height - CARD_HEIGHT);
	cardDraw(win, x, y, suit, value);
	XFlush(dpy);
    }
}

/*-----------------------------------------------------------------------+
|  PUBLIC FUNCTIONS                                                      |
+-----------------------------------------------------------------------*/

int
main(int argc, char *argv[])
{
    RANDOMIZE(time(NULL) + getpid());
    printf("initiating window\n");
    winInit();
    printf("initiating cards\n");
    cardInit(dpy, win);
    printf("doing something\n");
    doSomething();
    pause();
    printf("freeing cards\n");
    cardFinish();
    return 0;
}
